@@ -1,3 +1,5 @@
+import string
+
 from django.db import models
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
@@ -45,7 +47,7 @@ class Template(models.Model):
     content = models.TextField(unique=True, db_index=True)
 
     def __unicode__(self):
-        return self.name
+        return self.content
 
     def natural_key(self):
         return (self.content,)
@@ -58,8 +60,24 @@ class JournalManager(models.Manager):
     def for_object(self, obj):
         '''Return Journal records linked to this object.'''
         content_type = ContentType.objects.get_for_model(obj)
-        return self.filter(journalobjectdata__content_type=content_type,
-                journalobjectdata__object_id=obj.pk)
+        return self.filter(objectdata__content_type=content_type,
+                objectdata__object_id=obj.pk)
+
+    def for_objects(self, objects):
+        '''Return journal records linked to any of this objects.
+
+           All objects must have the same model.
+        '''
+        if not objects:
+            return self.none()
+        content_types = [ ContentType.objects.get_for_model(obj)
+                for obj in objects ]
+        if len(set(content_types)) != 1:
+            raise ValueError('objects must have of the same content type')
+        pks = [ obj.pk for obj in objects ]
+        return self.filter(
+                objectdata__content_type=content_types[0],
+                objectdata__object_id__in=pks)
 
     def for_tag(self, tag):
         '''Returns Journal records linked to this tag by their own tag or
@@ -111,6 +129,12 @@ class Journal(models.Model):
                         content_type=data.content_type, object_id=data.object_id)
         for data in self.stringdata_set.all():
             ctx[data.tag.name] = data.content
+        for text, field, format_spec, conversion in string.Formatter().parse(self.template.content):
+            if not field:
+                continue
+            splitted = field.split('.')
+            if splitted[0] not in ctx:
+                ctx[splitted[0]] = None
         return ctx
 
     def __unicode__(self):
